@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2007-2011 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,36 +11,36 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
+from __future__ import division
+
 if __name__ != '__main__':
     raise ImportError('This module is not intended for import')
 
-from djvu.sexpr import *
-from os import getpid
+import djvu.sexpr
+import os
 
-PROC_STATUS = '/proc/%d/status' % getpid()
-SCALE = dict(kB = 1024.0)
+proc_status = '/proc/{pid}/status'.format(pid=os.getpid())
+scale = dict(kB=1024)
 
-def mem_info(key = 'VmSize'):
+def mem_info(key='VmSize'):
     try:
-        file = open(PROC_STATUS)
+        file = open(proc_status)
         for line in file:
-            if line.startswith('%(key)s:' % locals()):
+            if line.startswith('{key}:'.format(key=key)):
                 _, value, unit = line.split(None, 3)
-                return float(value) * SCALE[unit]
+                return int(value) * scale[unit]
     finally:
         file.close()
 
-STEP = 1 << 17
-
 try:
     range = xrange
 except NameError:
     pass
 
-n = 0
+step = 1 << 17
 while True:
-    print('%.2fM' % mem_info())
-    [Expression(4) for i in range(STEP)]
-    break
+    mb = mem_info() / (1 << 20)
+    print('{mb:.2f}M'.format(mb=mb))
+    [djvu.sexpr.Expression(4) for i in range(step)]
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
