@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2008-2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,8 +11,6 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
-from __future__ import with_statement
-
 from djvu.const import *
 from djvu.sexpr import *
 
@@ -33,7 +31,7 @@ class test_text_zones():
     def test_type(self):
         for zone in self.zones:
             assert_equal(type(zone), TextZoneType)
-            assert_true(isinstance(zone, Symbol))
+            assert_is_instance(zone, Symbol)
 
     def test_repr(self):
         assert_repr(TEXT_ZONE_PAGE, '<djvu.const.TextZoneType: page>')
@@ -45,24 +43,24 @@ class test_text_zones():
         assert_repr(TEXT_ZONE_CHARACTER, '<djvu.const.TextZoneType: char>')
 
     def test_identity(self):
-        assert_true(TEXT_ZONE_PAGE is get_text_zone_type(Symbol('page')))
-        assert_true(TEXT_ZONE_COLUMN is get_text_zone_type(Symbol('column')))
-        assert_true(TEXT_ZONE_REGION is get_text_zone_type(Symbol('region')))
-        assert_true(TEXT_ZONE_PARAGRAPH is get_text_zone_type(Symbol('para')))
-        assert_true(TEXT_ZONE_LINE is get_text_zone_type(Symbol('line')))
-        assert_true(TEXT_ZONE_WORD is get_text_zone_type(Symbol('word')))
-        assert_true(TEXT_ZONE_CHARACTER is get_text_zone_type(Symbol('char')))
+        assert_is(TEXT_ZONE_PAGE, get_text_zone_type(Symbol('page')))
+        assert_is(TEXT_ZONE_COLUMN, get_text_zone_type(Symbol('column')))
+        assert_is(TEXT_ZONE_REGION, get_text_zone_type(Symbol('region')))
+        assert_is(TEXT_ZONE_PARAGRAPH, get_text_zone_type(Symbol('para')))
+        assert_is(TEXT_ZONE_LINE, get_text_zone_type(Symbol('line')))
+        assert_is(TEXT_ZONE_WORD, get_text_zone_type(Symbol('word')))
+        assert_is(TEXT_ZONE_CHARACTER, get_text_zone_type(Symbol('char')))
 
     def test_comparison1(self):
         assert_not_equal(TEXT_ZONE_PAGE, '')
         assert_not_equal(TEXT_ZONE_PAGE, 42)
-        with raises(TypeError, 'cannot compare text zone type with other object'):
+        with assert_raises_str(TypeError, 'cannot compare text zone type with other object'):
             TEXT_ZONE_PAGE < 42
-        with raises(TypeError, 'cannot compare text zone type with other object'):
+        with assert_raises_str(TypeError, 'cannot compare text zone type with other object'):
             TEXT_ZONE_PAGE <= 42
-        with raises(TypeError, 'cannot compare text zone type with other object'):
+        with assert_raises_str(TypeError, 'cannot compare text zone type with other object'):
             TEXT_ZONE_PAGE > 42
-        with raises(TypeError, 'cannot compare text zone type with other object'):
+        with assert_raises_str(TypeError, 'cannot compare text zone type with other object'):
             TEXT_ZONE_PAGE >= 42
 
     def test_comparison2(self):
@@ -79,4 +77,4 @@ class test_text_zones():
             ]
         )
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
