@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2007-2014 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,10 +11,11 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
-from __future__ import with_statement
-
 import collections
+import codecs
 import copy
+import errno
+import io
 import tempfile
 
 import pickle
@@ -24,6 +25,8 @@ except ImportError:
     cpickle = None
 
 from djvu.sexpr import *
+from djvu.sexpr import __version__
+from djvu.sexpr import _ExpressionIO
 
 from common import *
 
@@ -31,7 +34,7 @@ def assert_pickle_equal(obj):
     for pickle_module in pickle, cpickle:
         if pickle_module is None:
             continue
-        for protocol in 0, 1, 2:
+        for protocol in range(pickle.HIGHEST_PROTOCOL + 1):
             pickled_obj = pickle_module.dumps(obj, protocol=protocol)
             repickled_obj = pickle_module.loads(pickled_obj)
             assert_equal(obj, repickled_obj)
@@ -41,8 +44,9 @@ class test_int_expressions():
     def test_short(self):
         x = Expression(3)
         assert_repr(x, 'Expression(3)')
-        assert_true(x is Expression(x))
+        assert_is(x, Expression(x))
         assert_equal(x.value, 3)
+        assert_equal(x.lvalue, 3)
         assert_equal(str(x), '3')
         assert_repr(x, repr(Expression.from_string(str(x))))
         assert_equal(int(x), 3)
@@ -62,9 +66,9 @@ class test_int_expressions():
     def test_limits(self):
         assert_equal(Expression((1 << 29) - 1).value, (1 << 29) - 1)
         assert_equal(Expression(-1 << 29).value, -1 << 29)
-        with raises(ValueError, 'value not in range(-2 ** 29, 2 ** 29)'):
+        with assert_raises_str(ValueError, 'value not in range(-2 ** 29, 2 ** 29)'):
             Expression(1 << 29)
-        with raises(ValueError, 'value not in range(-2 ** 29, 2 ** 29)'):
+        with assert_raises_str(ValueError, 'value not in range(-2 ** 29, 2 ** 29)'):
             Expression((-1 << 29) - 1)
 
     def test_bool(self):
@@ -75,13 +79,23 @@ class test_int_expressions():
         x = Expression(42)
         assert_pickle_equal(x)
 
-def test_symbols():
+class test_float_expressions():
 
-    for name in 'eggs', u('ветчина'):
+    # TODO: float expressions are not implemented yet
+
+    def test_parse(self):
+        with assert_raises(ExpressionSyntaxError):
+            x = Expression.from_string('3.14')
+            if isinstance(x.value, Symbol):
+                raise ExpressionSyntaxError
+
+class test_symbols():
+
+    def t(self, name):
         symbol = Symbol(name)
         assert_equal(type(symbol), Symbol)
         assert_equal(symbol, Symbol(name))
-        assert_true(symbol is Symbol(name))
+        assert_is(symbol, Symbol(name))
         if py3k:
             assert_equal(str(symbol), name)
         else:
@@ -92,11 +106,24 @@ def test_symbols():
         assert_equal(hash(symbol), hash(name.encode('UTF-8')))
         assert_pickle_equal(symbol)
 
+    def test_ascii(self):
+        self.t('eggs')
+
+    def test_nonascii(self):
+        self.t(u('ветчина'))
+
+    def test_inequality(self):
+        assert_less(
+            Symbol('eggs'),
+            Symbol('ham'),
+        )
+
 def test_expressions():
     x = Expression(Symbol('eggs'))
     assert_repr(x, "Expression(Symbol('eggs'))")
-    assert_true(x is Expression(x))
+    assert_is(x, Expression(x))
     assert_equal(x.value, Symbol('eggs'))
+    assert_equal(x.lvalue, Symbol('eggs'))
     assert_equal(str(x), 'eggs')
     assert_repr(x, repr(Expression.from_string(str(x))))
     assert_equal(x, Expression(Symbol('eggs')))
@@ -108,8 +135,9 @@ def test_expressions():
 def test_string_expressions():
     x = Expression('eggs')
     assert_repr(x, "Expression('eggs')")
-    assert_true(x is Expression(x))
+    assert_is(x, Expression(x))
     assert_equal(x.value, 'eggs')
+    assert_equal(x.lvalue, 'eggs')
     assert_equal(str(x), '"eggs"')
     assert_repr(x, repr(Expression.from_string(str(x))))
     assert_equal(x, Expression('eggs'))
@@ -123,7 +151,7 @@ class test_unicode_expressions():
     def test1(self):
         x = Expression(u('eggs'))
         assert_repr(x, "Expression('eggs')")
-        assert_true(x is Expression(x))
+        assert_is(x, Expression(x))
 
     def test2(self):
         x = Expression(u('żółw'))
@@ -136,59 +164,62 @@ class test_list_expressions():
 
     def test1(self):
         x = Expression(())
-        assert_repr(x, "Expression(())")
+        assert_repr(x, "Expression([])")
         y = Expression(x)
-        assert_true(x is y)
+        assert_is(x, y)
         assert_equal(x.value, ())
+        assert_equal(x.lvalue, [])
         assert_equal(len(x), 0)
         assert_equal(bool(x), False)
         assert_equal(list(x), [])
 
     def test2(self):
         x = Expression([[1, 2], 3, [4, 5, Symbol('baz')], ['quux']])
-        assert_repr(x, "Expression(((1, 2), 3, (4, 5, Symbol('baz')), ('quux',)))")
+        assert_repr(x, "Expression([[1, 2], 3, [4, 5, Symbol('baz')], ['quux']])")
         y = Expression(x)
         assert_repr(y, repr(x))
         assert_false(x is y)
         assert_equal(x.value, ((1, 2), 3, (4, 5, Symbol('baz')), ('quux',)))
+        assert_equal(x.lvalue, [[1, 2], 3, [4, 5, Symbol('baz')], ['quux']])
         assert_equal(str(x), '((1 2) 3 (4 5 baz) ("quux"))')
         assert_repr(x, repr(Expression.from_string(str(x))))
         assert_equal(len(x), 4)
         assert_equal(bool(x), True)
         assert_equal(tuple(x), (Expression((1, 2)), Expression(3), Expression((4, 5, Symbol('baz'))), Expression(('quux',))))
-        with raises(TypeError, 'key must be an integer or a slice'):
+        with assert_raises_str(TypeError, 'key must be an integer or a slice'):
             x[object()]
         assert_equal(x[1], Expression(3))
         assert_equal(x[-1][0], Expression('quux'))
-        with raises(IndexError, 'list index of out range'):
+        with assert_raises_str(IndexError, 'list index of out range'):
             x[6]
-        with raises(IndexError, 'list index of out range'):
+        with assert_raises_str(IndexError, 'list index of out range'):
             x[-6]
         assert_equal(x[:].value, x.value)
-        assert_repr(x[1:], "Expression((3, (4, 5, Symbol('baz')), ('quux',)))")
-        assert_repr(x[-2:], "Expression(((4, 5, Symbol('baz')), ('quux',)))")
+        assert_equal(x[:].lvalue, x.lvalue)
+        assert_repr(x[1:], "Expression([3, [4, 5, Symbol('baz')], ['quux']])")
+        assert_repr(x[-2:], "Expression([[4, 5, Symbol('baz')], ['quux']])")
         x[-2:] = 4, 5, 6
-        assert_repr(x, 'Expression(((1, 2), 3, 4, 5, 6))')
+        assert_repr(x, 'Expression([[1, 2], 3, 4, 5, 6])')
         x[0] = 2
-        assert_repr(x, 'Expression((2, 3, 4, 5, 6))')
+        assert_repr(x, 'Expression([2, 3, 4, 5, 6])')
         x[:] = (1, 3, 5)
-        assert_repr(x, 'Expression((1, 3, 5))')
+        assert_repr(x, 'Expression([1, 3, 5])')
         x[3:] = 7,
-        assert_repr(x, 'Expression((1, 3, 5, 7))')
-        with raises(NotImplementedError, 'only [n:] slices are supported'):
+        assert_repr(x, 'Expression([1, 3, 5, 7])')
+        with assert_raises_str(NotImplementedError, 'only [n:] slices are supported'):
             x[object():]
-        with raises(NotImplementedError, 'only [n:] slices are supported'):
+        with assert_raises_str(NotImplementedError, 'only [n:] slices are supported'):
             x[:2]
-        with raises(NotImplementedError, 'only [n:] slices are supported'):
+        with assert_raises_str(NotImplementedError, 'only [n:] slices are supported'):
             x[object():] = []
-        with raises(NotImplementedError, 'only [n:] slices are supported'):
+        with assert_raises_str(NotImplementedError, 'only [n:] slices are supported'):
             x[:2] = []
-        with raises(TypeError, 'can only assign a list expression'):
+        with assert_raises_str(TypeError, 'can only assign a list expression'):
             x[:] = 0
         assert_equal(x, Expression((1, 3, 5, 7)))
         assert_not_equal(x, Expression((2, 4, 6)))
         assert_not_equal(x, (1, 3, 5, 7))
-        with raises(TypeError, "unhashable type: 'ListExpression'"):
+        with assert_raises_str(TypeError, "unhashable type: 'ListExpression'"):
             hash(x)
 
     def test_insert(self):
@@ -196,16 +227,16 @@ class test_list_expressions():
         expr = Expression(())
         for pos in [-8, 4, 6, -5, -7, 5, 7, 2, -3, 8, 10, -2, 1, -9, -10, -4, -6, 0, 9, 3, -1]:
             lst.insert(pos, pos)
-            assert_true(expr.insert(pos, pos) is None)
+            assert_is(expr.insert(pos, pos), None)
             assert_equal(expr, Expression(lst))
-            assert_equal(list(expr.value), lst)
+            assert_equal(expr.lvalue, lst)
 
     def test_append(self):
         expr = Expression(())
         for i in range(10):
-            assert_true(expr.append(i) is None)
+            assert_is(expr.append(i), None)
             assert_equal(expr, Expression(range(i + 1)))
-            assert_equal(list(expr.value), list(range(i + 1)))
+            assert_equal(expr.lvalue, list(range(i + 1)))
 
     def test_extend(self):
         lst = []
@@ -214,8 +245,8 @@ class test_list_expressions():
             lst.extend(ext)
             expr.extend(ext)
             assert_equal(expr, Expression(lst))
-            assert_equal(list(expr.value), lst)
-        with raises(TypeError, "'int' object is not iterable"):
+            assert_equal(expr.lvalue, lst)
+        with assert_raises_str(TypeError, "'int' object is not iterable"):
             expr.extend(0)
 
     def test_inplace_add(self):
@@ -225,16 +256,16 @@ class test_list_expressions():
             lst += ext
             expr += ext
             assert_equal(expr, Expression(lst))
-            assert_equal(list(expr.value), lst)
-        assert_true(expr is expr0)
-        with raises(TypeError, "'int' object is not iterable"):
+            assert_equal(expr.lvalue, lst)
+        assert_is(expr, expr0)
+        with assert_raises_str(TypeError, "'int' object is not iterable"):
             expr += 0
 
     def test_pop(self):
         expr = Expression([0, 1, 2, 3, 4, 5, 6])
         assert_equal(expr.pop(0), Expression(0))
         assert_equal(expr, Expression([1, 2, 3, 4, 5, 6]))
-        with raises(IndexError, 'pop index of out range'):
+        with assert_raises_str(IndexError, 'pop index of out range'):
             expr.pop(6)
         assert_equal(expr.pop(5), Expression(6))
         assert_equal(expr, Expression([1, 2, 3, 4, 5]))
@@ -246,17 +277,17 @@ class test_list_expressions():
         assert_equal(expr, Expression([1, 4]))
         expr.pop()
         expr.pop()
-        with raises(IndexError, 'pop from empty list'):
+        with assert_raises_str(IndexError, 'pop from empty list'):
             expr.pop()
         for i in range(-2, 3):
-            with raises(IndexError, 'pop from empty list'):
+            with assert_raises_str(IndexError, 'pop from empty list'):
                 expr.pop(i)
 
     def test_delitem(self):
         expr = Expression([0, 1, 2, 3, 4, 5, 6])
         del expr[0]
         assert_equal(expr, Expression([1, 2, 3, 4, 5, 6]))
-        with raises(IndexError, 'pop index of out range'):
+        with assert_raises_str(IndexError, 'pop index of out range'):
             expr.pop(6)
         del expr[5]
         assert_equal(expr, Expression([1, 2, 3, 4, 5]))
@@ -271,14 +302,14 @@ class test_list_expressions():
         del expr[:]
         assert_equal(expr, Expression([]))
         for i in range(-2, 3):
-            with raises(IndexError, 'pop from empty list'):
+            with assert_raises_str(IndexError, 'pop from empty list'):
                 del expr[i]
 
     def test_remove(self):
         expr = Expression([0, 1, 2, 3, 4, 5, 6])
         expr.remove(Expression(0))
         assert_equal(expr, Expression([1, 2, 3, 4, 5, 6]))
-        with raises(IndexError, 'item not in list'):
+        with assert_raises_str(IndexError, 'item not in list'):
             expr.remove(Expression(0))
         expr.remove(Expression(6))
         assert_equal(expr, Expression([1, 2, 3, 4, 5]))
@@ -290,22 +321,22 @@ class test_list_expressions():
         assert_equal(expr, Expression([1, 4]))
         expr.remove(Expression(4))
         expr.remove(Expression(1))
-        with raises(IndexError, 'item not in list'):
+        with assert_raises_str(IndexError, 'item not in list'):
             expr.remove(Expression(-1))
 
     def test_contains(self):
         expr = Expression(())
-        assert_false(Expression(42) in expr)
+        assert_not_in(Expression(42), expr)
         lst = (1, 2, 3)
         expr = Expression(lst)
         for x in lst:
-            assert_false(x in expr)
-            assert_true(Expression(x) in expr)
-        assert_false(Expression(max(lst) + 1) in expr)
+            assert_not_in(x, expr)
+            assert_in(Expression(x), expr)
+        assert_not_in(Expression(max(lst) + 1), expr)
 
     def test_index(self):
         expr = Expression(())
-        with raises(ValueError, 'value not in list'):
+        with assert_raises_str(ValueError, 'value not in list'):
             expr.index(Expression(42))
         lst = [1, 2, 3]
         expr = Expression(lst)
@@ -313,7 +344,7 @@ class test_list_expressions():
             i = lst.index(x)
             j = expr.index(Expression(x))
             assert_equal(i, j)
-        with raises(ValueError, 'value not in list'):
+        with assert_raises_str(ValueError, 'value not in list'):
             expr.index(Expression(max(lst) + 1))
 
     def test_count(self):
@@ -335,7 +366,7 @@ class test_list_expressions():
                 Expression(reversed(expr)).value,
                 tuple(reversed(lst))
             )
-            assert_true(expr.reverse() is None)
+            assert_is(expr.reverse(), None)
             assert_equal(
                 expr,
                 Expression(reversed(lst))
@@ -349,166 +380,237 @@ class test_list_expressions():
         x = Expression([1, [2], 3])
         y = Expression(x)
         x[1][0] = 0
-        assert_repr(x, 'Expression((1, (0,), 3))')
-        assert_repr(y, 'Expression((1, (0,), 3))')
+        assert_repr(x, 'Expression([1, [0], 3])')
+        assert_repr(y, 'Expression([1, [0], 3])')
         x[1] = 0
-        assert_repr(x, 'Expression((1, 0, 3))')
-        assert_repr(y, 'Expression((1, (0,), 3))')
+        assert_repr(x, 'Expression([1, 0, 3])')
+        assert_repr(y, 'Expression([1, [0], 3])')
 
     def test_copy2(self):
         x = Expression([1, [2], 3])
         y = copy.copy(x)
         x[1][0] = 0
-        assert_repr(x, 'Expression((1, (0,), 3))')
-        assert_repr(y, 'Expression((1, (0,), 3))')
+        assert_repr(x, 'Expression([1, [0], 3])')
+        assert_repr(y, 'Expression([1, [0], 3])')
         x[1] = 0
-        assert_repr(x, 'Expression((1, 0, 3))')
-        assert_repr(y, 'Expression((1, (0,), 3))')
+        assert_repr(x, 'Expression([1, 0, 3])')
+        assert_repr(y, 'Expression([1, [0], 3])')
 
     def test_copy3(self):
         x = Expression([1, [2], 3])
         y = copy.deepcopy(x)
         x[1][0] = 0
-        assert_repr(x, 'Expression((1, (0,), 3))')
-        assert_repr(y, 'Expression((1, (2,), 3))')
+        assert_repr(x, 'Expression([1, [0], 3])')
+        assert_repr(y, 'Expression([1, [2], 3])')
         x[1] = 0
-        assert_repr(x, 'Expression((1, 0, 3))')
-        assert_repr(y, 'Expression((1, (2,), 3))')
+        assert_repr(x, 'Expression([1, 0, 3])')
+        assert_repr(y, 'Expression([1, [2], 3])')
 
-    if sys.version_info >= (2, 6):
-        def test_abc(self):
-            x = Expression(())
-            assert_true(isinstance(x, collections.MutableSequence))
-            assert_true(isinstance(iter(x), collections.Iterator))
+    def test_abc(self):
+        x = Expression(())
+        assert_is_instance(x, collections.MutableSequence)
+        assert_is_instance(iter(x), collections.Iterator)
 
     def test_pickle(self):
         for lst in (), (1, 2, 3), (1, (2, 3)):
             x = Expression(lst)
             assert_pickle_equal(x)
 
-def strip_line_numbers_from_traceback(s):
-    s = re.sub('(?<=[.]c):[0-9]+(?=[)])', '', s)
-    s = re.sub(', line [0-9]+(?=, )', '', s)
-    return s
-
 class test_expression_parser():
 
     def test_badstring(self):
-        with raises(ExpressionSyntaxError):
+        with assert_raises(ExpressionSyntaxError):
             Expression.from_string('(1')
 
     def test_attr_from_file(self):
-        assert getattr(Expression, 'from_file', None) is None
+        assert_is(getattr(Expression, 'from_file', None), None)
 
     def test_bad_io(self):
-        stderr = StringIO()
-        with interim(sys, stderr=stderr):
-            with raises(ExpressionSyntaxError):
-                Expression.from_stream(42)
-        stderr = strip_line_numbers_from_traceback(stderr.getvalue())
-        stderr = stderr.replace(
-            '"sexpr.pyx"', # Cython < 0.21
-            '"djvu/sexpr.pyx"' # Cython ≥ 0.21
-        )
-        assert_multi_line_equal(stderr, '''\
-Unhandled exception (42)
-Traceback (most recent call last):
-  File "djvu/sexpr.pyx", in djvu.sexpr._myio_getc (djvu/sexpr.c)
-AttributeError: 'int' object has no attribute 'read'
+        with assert_raises_str(AttributeError, "'int' object has no attribute 'read'"):
+            Expression.from_stream(42)
 
-''')
+    def test_bad_file_io(self):
+        with open('/proc/self/mem') as fp:
+            with assert_raises(IOError) as ecm:
+                Expression.from_stream(fp)
+        assert_equal(ecm.exception.errno, errno.EIO)
+
+    if py3k:
+        def test_bad_unicode_io(self):
+            fp = StringIO(chr(0xD800))
+            with assert_raises(UnicodeEncodeError):
+                Expression.from_stream(fp)
+
+class test_expression_parser_ascii():
+
+    expr = '(eggs) (ham)'
+    repr = ["Expression([Symbol('eggs')])", "Expression([Symbol('ham')])"]
+
+    def _test_fp(self, fp):
+        def read():
+            return Expression.from_stream(fp)
+        x = read()
+        assert_repr(x, self.repr[0])
+        x = read()
+        assert_repr(x, self.repr[1])
+        with assert_raises(ExpressionSyntaxError):
+            x = read()
 
     def test_stringio(self):
-        fp = StringIO('(eggs) (ham)')
-        def read():
-            return Expression.from_stream(fp)
-        x = read()
-        assert_repr(x, "Expression((Symbol('eggs'),))")
-        x = read()
-        assert_repr(x, "Expression((Symbol('ham'),))")
-        with raises(ExpressionSyntaxError):
-            x = read()
+        fp = StringIO(self.expr)
+        self._test_fp(fp)
 
-    def test_fileio_text(self):
-        fp = tempfile.TemporaryFile(mode='w+t')
-        def read():
-            return Expression.from_stream(fp)
-        if not py3k:
-            assert_equal(type(fp), file)
-        fp.write('(eggs) (ham)')
-        fp.flush()
-        fp.seek(0)
-        x = read()
-        assert_repr(x, "Expression((Symbol('eggs'),))")
-        x = read()
-        assert_repr(x, "Expression((Symbol('ham'),))")
-        with raises(ExpressionSyntaxError):
-            x = read()
+    def test_bytesio(self):
+        fp = io.BytesIO(b(self.expr))
+        self._test_fp(fp)
 
-    def test_fileio_binary(self):
-        fp = tempfile.TemporaryFile(mode='w+b')
-        def read():
-            return Expression.from_stream(fp)
-        if not py3k:
-            assert_equal(type(fp), file)
-        fp.write(b('(eggs) (ham)'))
-        fp.flush()
-        fp.seek(0)
-        x = read()
-        assert_repr(x, "Expression((Symbol('eggs'),))")
-        x = read()
-        assert_repr(x, "Expression((Symbol('ham'),))")
-        with raises(ExpressionSyntaxError):
-            x = read()
+    def test_file_io_text(self):
+        if py3k:
+            fp = tempfile.TemporaryFile(mode='w+t', encoding='UTF-16-LE')
+        else:
+            fp = tempfile.TemporaryFile(mode='w+t')
+        with fp:
+            fp.write(self.expr)
+            fp.flush()
+            fp.seek(0)
+            self._test_fp(fp)
+
+    def test_codecs_io(self):
+        with tempfile.NamedTemporaryFile(mode='w+b') as bfp:
+            with codecs.open(bfp.name, mode='w+', encoding='UTF-16-LE') as fp:
+                fp.write(u(self.expr))
+                fp.seek(0)
+                self._test_fp(fp)
+
+    def test_file_io_binary(self):
+        with tempfile.TemporaryFile(mode='w+b') as fp:
+            fp.write(b(self.expr))
+            fp.flush()
+            fp.seek(0)
+            self._test_fp(fp)
+
+class test_expression_parser_nonascii(test_expression_parser_ascii):
+
+    expr = '"jeż" "żółw"'
+    repr = [r"Expression('jeż')", r"Expression('żółw')"]
+    if not py3k:
+        repr = [s.decode('ISO-8859-1').encode('ASCII', 'backslashreplace') for s in repr]
 
 class test_expression_writer():
 
-    expr = Expression([Symbol('eggs'), Symbol('ham')])
-    repr = '(eggs ham)'
-
     def test_bad_io(self):
-        stderr = StringIO()
-        with interim(sys, stderr=stderr):
-            self.expr.print_into(42)
-        stderr = strip_line_numbers_from_traceback(stderr.getvalue())
-        stderr = stderr.replace(
-            '"sexpr.pyx"', # Cython < 0.21
-            '"djvu/sexpr.pyx"' # Cython ≥ 0.21
-        )
-        expected_stderr = '''\
-Unhandled exception (42)
-Traceback (most recent call last):
-  File "djvu/sexpr.pyx", in djvu.sexpr._myio_puts (djvu/sexpr.c)
-AttributeError: 'int' object has no attribute 'write'
+        expr = Expression(23)
+        with assert_raises_str(AttributeError, "'int' object has no attribute 'write'"):
+            expr.print_into(42)
 
-'''
-        expected_stderr *= len(stderr) // len(expected_stderr)
-        assert_multi_line_equal(stderr, expected_stderr)
+    def test_bad_file_io(self):
+        ecm = None
+        fp = open('/dev/full', 'w', buffering=2)
+        expr = Expression(23)
+        try:
+            with assert_raises(IOError) as ecm:
+                for i in range(10000):
+                    expr.print_into(fp)
+        finally:
+            try:
+                fp.close()
+            except IOError:
+                if ecm is None:
+                    raise
+        assert_equal(ecm.exception.errno, errno.ENOSPC)
 
-    def test_stringio(self):
+    def test_reentrant(self):
+        if not _ExpressionIO._reentrant:
+            raise SkipTest('this test requires DjVuLibre >= 3.5.26')
+        class File(object):
+            def write(self, s):
+                expr.as_string()
+        expr = Expression(23)
+        fp = File()
+        expr.print_into(fp)
+
+class test_expression_writer_ascii():
+
+    expr = Expression([Symbol('eggs'), Symbol('ham')])
+    repr = urepr = '(eggs ham)'
+
+    def test_stringio_7(self):
         fp = StringIO()
         self.expr.print_into(fp)
         assert_equal(fp.getvalue(), self.repr)
 
-    def test_fileio_text(self):
-        fp = tempfile.TemporaryFile(mode='w+t')
-        if not py3k and os.name == 'posix':
-            assert_equal(type(fp), file)
-        self.expr.print_into(fp)
-        fp.seek(0)
-        assert_equal(fp.read(), self.repr)
+    def test_stringio_8(self):
+        fp = StringIO()
+        self.expr.print_into(fp, escape_unicode=False)
+        assert_equal(fp.getvalue(), self.urepr)
 
-    def test_fileio_binary(self):
-        fp = tempfile.TemporaryFile(mode='w+b')
-        if not py3k and os.name == 'posix':
-            assert_equal(type(fp), file)
+    def test_bytesio_7(self):
+        fp = io.BytesIO()
         self.expr.print_into(fp)
-        fp.seek(0)
-        assert_equal(fp.read(), b(self.repr))
+        assert_equal(fp.getvalue(), b(self.repr))
 
-class test_expression_writer_nonascii(test_expression_writer):
+    def test_bytesio_8(self):
+        fp = io.BytesIO()
+        self.expr.print_into(fp, escape_unicode=False)
+        assert_equal(fp.getvalue(), b(self.urepr))
+
+    def test_file_io_text_7(self):
+        with tempfile.TemporaryFile(mode='w+t') as fp:
+            self.expr.print_into(fp)
+            fp.seek(0)
+            assert_equal(fp.read(), self.repr)
+
+    def test_file_io_text_8(self):
+        if py3k:
+            fp = tempfile.TemporaryFile(mode='w+t', encoding='UTF-16-LE')
+        else:
+            fp = tempfile.TemporaryFile(mode='w+t')
+        with fp:
+            self.expr.print_into(fp, escape_unicode=False)
+            fp.seek(0)
+            assert_equal(fp.read(), self.urepr)
+
+    def test_codecs_io_text_7(self):
+        with tempfile.NamedTemporaryFile(mode='w+b') as bfp:
+            with codecs.open(bfp.name, mode='w+', encoding='UTF-16-LE') as fp:
+                self.expr.print_into(fp)
+                fp.seek(0)
+                assert_equal(fp.read(), self.repr)
+
+    def test_codecs_io_text_8(self):
+        with tempfile.NamedTemporaryFile(mode='w+b') as bfp:
+            with codecs.open(bfp.name, mode='w+', encoding='UTF-16-LE') as fp:
+                self.expr.print_into(fp, escape_unicode=False)
+                fp.seek(0)
+                assert_equal(fp.read(), u(self.urepr))
+
+    def test_file_io_binary_7(self):
+        with tempfile.TemporaryFile(mode='w+b') as fp:
+            self.expr.print_into(fp)
+            fp.seek(0)
+            assert_equal(fp.read(), b(self.repr))
+
+    def test_file_io_binary_8(self):
+        with tempfile.TemporaryFile(mode='w+b') as fp:
+            self.expr.print_into(fp, escape_unicode=False)
+            fp.seek(0)
+            assert_equal(fp.read(), b(self.urepr))
+
+    def test_as_string_7(self):
+        s = self.expr.as_string()
+        assert_equal(s, self.repr)
+
+    def test_as_string_8(self):
+        s = self.expr.as_string(escape_unicode=False)
+        assert_equal(s, self.urepr)
+
+class test_expression_writer_nonascii(test_expression_writer_ascii):
 
     expr = Expression(u('żółw'))
     repr = r'"\305\274\303\263\305\202w"'
+    urepr = r'"żółw"'
 
-# vim:ts=4 sw=4 et
+def test_version():
+    assert_is_instance(__version__, str)
+
+# vim:ts=4 sts=4 sw=4 et
